@@ -33,7 +33,9 @@ setClass(Class = "MxExpectationRAM",
 	    between = "MxOptionalCharOrNumber",
 	    verbose = "integer",
 	    .rampart = "integer",
-	    .forceSingleGroup = "logical"
+	    .forceSingleGroup = "logical",
+	    .ignoreDefVarsHack = "logical", #remove TODO
+	    .identifyZeroVarPred = "logical"
 	),
 	contains = "BaseExpectationNormal")
 
@@ -55,6 +57,8 @@ setMethod("initialize", "MxExpectationRAM",
 		.Object@verbose <- verbose
 		.Object@.rampart <- as.integer(NA)
 		.Object@.forceSingleGroup <- FALSE
+		.Object@.ignoreDefVarsHack <- FALSE  # remove TODO
+		.Object@.identifyZeroVarPred <- TRUE
 		return(.Object)
 	}
 )
@@ -112,6 +116,11 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 		if (.hasSlot(.Object, "between") && length(.Object@between)) {
 			.Object@between <- sapply(.Object@between, function(bName) {
 				zMat <- flatModel[[ bName ]]
+				if (is.null(zMat)) {
+					msg <- paste("Level transition matrix", omxQuotes(bName),
+						     "listed in", omxQuotes(name), "is not found")
+					stop(msg, call. = FALSE)
+				}
 				expName <- paste0(zMat@joinModel, imxSeparatorChar, 'expectation')
 				upperA <- flatModel[[ flatModel@expectations[[ expName ]]$A ]]
 				lowerA <- flatModel[[ aMatrix ]]
