@@ -25,10 +25,10 @@ for (retry in 1:2) {
   if (retry == 2) n2 <- mxModel(n2, plan)
   n2Fit <- mxRun(n2)
   
-  omxCheckCloseEnough(n2Fit$output$fit, 80.866, .01)
-  omxCheckCloseEnough(n2Fit$cov$values, dataCov, 1e-4)
+  omxCheckCloseEnough(n2Fit$output$fit, 81.216, .01)
+  omxCheckCloseEnough(n2Fit$cov$values, (34/35) * dataCov, 1e-4)
   omxCheckCloseEnough(c(n2Fit$mean$values), dataMeans, 1e-4)
   omxCheckCloseEnough(log(n2Fit$output$conditionNumber), 1.63, .2)
   omxCheckCloseEnough(c(n2Fit$output$standardErrors),
-                      c(0.243, 0.213, 0.364, 0.169, 0.207), .01)
+                      c(0.232, 0.203, 0.348, 0.166, 0.204), .01)
 }
