@@ -9,7 +9,7 @@ m1 <- mxModel("m1", type="RAM",
 	manifestVars = manifests, latentVars   = latents,
 	mxPath(from = latents, to = manifests),
 	mxPath(from = manifests, arrows = 2), # manifest residuals 
-	mxPath(from = latents, arrows = 2, free = FALSE, values = 1), # latent fixed@1
+	mxPath(from = latents, arrows = 2, free = FALSE, values = 1), # latent fixed at 1
 	mxPath(from = "one", to = manifests, arrows = 1), # manifest means
 	mxData(myFADataRaw[1:100, manifests], type = "raw")
 )
