@@ -179,20 +179,7 @@ void omxFitFunctionPreoptimize(omxFitFunction *off, FitContext *fc)
 void ComputeFit(const char *callerName, omxMatrix *fitMat, int want, FitContext *fc)
 {
 	bool doFit = want & FF_COMPUTE_FIT;
-	R_CheckUserInterrupt();
-
-#pragma omp atomic
-	++Global->computeCount; // could avoid lock by keeping in FitContext
-
-	// old version of openmp can't do this as part of the atomic instruction
-	int evaluation = Global->computeCount;
-
-	if (doFit) {
-		if (OMX_DEBUG) {
-			mxLog("%s: starting evaluation %d, want %d", fitMat->name(), evaluation, want);
-		}
-		Global->checkpointPrefit(callerName, fc, fc->est, false);
-	}
+	fc->incrComputeCount();
 	omxFitFunction *ff = fitMat->fitFunction;
 	if (ff) {
 		omxFitFunctionComputeAuto(ff, want, fc);
@@ -206,9 +193,9 @@ void ComputeFit(const char *callerName, omxMatrix *fitMat, int want, FitContext
 		if (std::isfinite(fc->fit)) {
 			fc->resetIterationError();
 		}
-		Global->checkpointPostfit(fc);
+		Global->checkpointPostfit(callerName, fc, fc->est, false);
 		if (OMX_DEBUG) {
-			mxLog("%s: completed evaluation %d, fit=%f", fitMat->name(), evaluation, fc->fit);
+			mxLog("%s: completed evaluation, fit=%.12g", fitMat->name(), fc->fit);
 		}
 	}
 }
@@ -371,8 +358,9 @@ void loglikelihoodCIFun(omxFitFunction *ff, int want, FitContext *fc)
 
 	omxFitFunctionCompute(fitMat->fitFunction, FF_COMPUTE_FIT, fc);
 	const double fit = totalLogLikelihood(fitMat);
-	omxRecompute(CI->matrix, fc);
-	double CIElement = omxMatrixElement(CI->matrix, CI->row, CI->col);
+	omxMatrix *ciMatrix = CI->getMatrix(fitMat->currentState);
+	omxRecompute(ciMatrix, fc);
+	double CIElement = omxMatrixElement(ciMatrix, CI->row, CI->col);
 	omxResizeMatrix(fitMat, 1, 1);
 
 	if (!std::isfinite(fit) || !std::isfinite(CIElement)) {
